"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useCramList = void 0;
var _react = require("react");
var _useIsomorphicLayoutEffect = require("../useIsomorphicLayoutEffect");
var _useWindowResize = require("../useWindowResize");
function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }
function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }
function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }
function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }
var closureClassName = function () {
  var count = 0;
  return function () {
    count++;
    return "usecramlist-cache-".concat(count);
  };
}();
var isClient = typeof window !== 'undefined';
var paddingsSum = function paddingsSum(elem) {
  if (isClient) {
    // для SSR проверка наличия window
    var computed = window.getComputedStyle(elem);
    return parseInt(computed.getPropertyValue('padding-left'), 10) + parseInt(computed.getPropertyValue('padding-right'), 10);
  }
  return 0;
};
var marginsSum = function marginsSum(elem) {
  if (isClient) {
    // для SSR проверка наличия window
    var computed = window.getComputedStyle(elem);
    return parseInt(computed.getPropertyValue('margin-left'), 10) + parseInt(computed.getPropertyValue('margin-right'), 10);
  }
  return 0;
};
var resizeHandler = function resizeHandler(containerRef, extraComponentRef, elementsRef, items, setCount, startWith) {
  if (containerRef.current && extraComponentRef && extraComponentRef.current) {
    var sum = 0;
    var _count = 0;
    var main = elementsRef.refs;
    var extraElement = extraComponentRef.current;
    var extra = extraElement.offsetWidth || main[items.length].offsetWidth;
    var paddingsWidth = paddingsSum(containerRef.current);
    var parentWidth = containerRef.current.offsetWidth - paddingsWidth;
    extraElement.style.visibility = 'hidden'; // сокрытие компонента заместитиеля
    extraElement.style.position = 'fixed'; // убирем его что бы не занимал места

    // '-1' -> вычитаем блок "Еще"
    for (var idx = 0; idx < main.length - 1; idx++) {
      main[idx].style.width = ''; // сброс
      main[idx].style.display = ''; // сброс
    }
    for (var _idx = 0; _idx < main.length - 1; _idx++) {
      if (main[_idx] === extraComponentRef.current) {
        continue;
      }
      main[_idx].style.display = '';
      var prevSum = sum;
      sum += main[_idx].offsetWidth + marginsSum(main[_idx]);
      if (startWith === 0 && sum > parentWidth && _idx + 1 > startWith) {
        _count++;
        main[_idx].style.display = 'none';
      }
      if (startWith === 0 && _count && _idx && _idx - 1 > startWith && main[_idx - 1].style.display !== 'none' && prevSum + extra + marginsSum(extraElement) + marginsSum(extraElement) > parentWidth) {
        _count++;
        main[_idx - 1].style.display = 'none';
      }
    }

    // Код отвечающий за то чтобы всегда показывать те элементы которые меньше чем startWith
    // '-1' -> вычитаем блок "Еще"
    if (startWith > 0 && main.length - 1 > startWith) {
      for (var _idx2 = startWith; _idx2 < main.length - 1; _idx2++) {
        main[_idx2].style.display = 'none';
        _count++;
      }
    }
    if (_count) {
      extraElement.style.visibility = 'visible';
      extraElement.style.position = '';
    }
    setCount(_count);
  }
};
var useCramList = function useCramList(items) {
  var startWith = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  var containerRef = (0, _react.useRef)(null);
  var extraComponentRef = (0, _react.useRef)(null);
  var elementsRef = (0, _react.useMemo)(function () {
    return {
      refs: []
    };
  }, []);
  var additionalClass = (0, _react.useMemo)(function () {
    return closureClassName();
  }, []);
  var styleSheet = (0, _react.useRef)();
  var _useState = (0, _react.useState)(items.length),
    _useState2 = _slicedToArray(_useState, 2),
    count = _useState2[0],
    setCount = _useState2[1];
  (0, _useWindowResize.useWindowResize)(function () {
    return resizeHandler(containerRef, extraComponentRef, elementsRef, items, setCount, startWith);
  });
  (0, _useIsomorphicLayoutEffect.useIsomorphicLayoutEffect)(function () {
    if (containerRef.current && isClient) {
      // Проверяем document для SSR
      // Делаем детей невидимыми что бы избежать мерцания, и избавится от необходимости определять стили вручную
      var styleEl = document.createElement('style');
      document.head.appendChild(styleEl);
      styleSheet.current = styleEl.sheet;
      containerRef.current.classList.add(additionalClass);
      return function () {
        document.head.removeChild(styleEl);
      };
    }
    return function () {};
  }, [additionalClass]);
  (0, _useIsomorphicLayoutEffect.useIsomorphicLayoutEffect)(function () {
    if (containerRef.current && extraComponentRef.current) {
      var _styleSheet$current, _styleSheet$current2;
      var children = Array.from(containerRef.current.children);
      elementsRef.refs = children;
      (_styleSheet$current = styleSheet.current) === null || _styleSheet$current === void 0 ? void 0 : _styleSheet$current.insertRule(".".concat(additionalClass, " > * {visibility: hidden;}"), 0);
      resizeHandler(containerRef, extraComponentRef, elementsRef, items, setCount, startWith);
      (_styleSheet$current2 = styleSheet.current) === null || _styleSheet$current2 === void 0 ? void 0 : _styleSheet$current2.deleteRule(0);
    }
  }, [additionalClass, elementsRef, items, startWith]);

  // Правильно, конечно, указать возвращаемый тип во время объявления функции, но
  // будет получена ошибка Parsing error: Cannot read properties of undefined (reading 'map')
  // Вероятней всего, это связано со старой версией линтера
  return [containerRef, extraComponentRef, count];
};
exports.useCramList = useCramList;