/* eslint-disable @typescript-eslint/no-unused-expressions */
/* eslint-disable no-lonely-if */
import { useEffect, useRef, useCallback } from 'react';
import { getScrollParent, getScrollParents, isElementInViewport } from '../getScrollParent';
import { useDebounce } from '../useDebounce';
// В коде используются разные расчеты для двух случаев позицианирования
// 1 - fixed
// 2 - absolute
// разные расчеты требуются из-за того, что в случае с fixed и getBoundingClientRect все координаты считаются
// относительно левого верхнего угла, т.е. right = left + width .
// В случае с absolute left = 0 и right = 0 указывают отступ от левого и правого краев родительского (пближайшего relative) элемента.
// Если возникают ошибки в позиционировании - следует проверить методы расчетов и обратить внимание на тип позиционирования.

// Устанавливаем правильную начальную позицию относительно родителя
var setInitPositionByParent = function setInitPositionByParent(element, parentRect, float, align, margins, fixed) {
  // Если элемент выпадает по горизонтали
  if (align === 'horizontal' || align === 'left' || align === 'right') {
    // Приоритет выпадания справа налево (элемент появится слева от родителя)
    if (align === 'right' || align === 'horizontal' && float === 'end') {
      var right = fixed ? parentRect.right - element.offsetWidth - margins.x : -element.offsetWidth - margins.x;
      element.style.right = "".concat(right, "px");
    } else {
      var left = fixed ? parentRect.left - element.offsetWidth - margins.x : -element.offsetWidth - margins.x;
      element.style.left = "".concat(left, "px");
    }
    if (float === 'start') {
      // Элемент установится по нижнему краю с отступом
      var top = fixed ? parentRect.top + margins.y : margins.y;
      element.style.top = "".concat(top, "px");
      element.style.bottom = ''; // сброс неопределенности, так как позиционные параметры более приоритетны, чем параметры размеров
    }
    if (float === 'center') {
      // Элемент установится по центру без отступа, в зависимости от соотношения размеров родителя и списка в плоскости соприкосновения
      var shift = (parentRect.height - element.offsetHeight) / 2;
      var _top = fixed ? parentRect.top + shift : shift;
      element.style.top = "".concat(_top, "px");
      element.style.bottom = ''; // сброс неопределенности, так как позиционные параметры более приоритетны, чем параметры размеров
    }
    if (float === 'end') {
      // Элемент установится по верхнему краю с отступом
      if (fixed) {
        element.style.top = "".concat(parentRect.bottom - element.offsetHeight - margins.y, "px");
        element.style.bottom = ''; // --
      } else {
        element.style.top = "".concat(margins.y, "px");
        element.style.bottom = ''; // --
      }
    }
  }
  if (align === 'vertical' || align === 'top' || align === 'bottom') {
    // Приоритет выпадания сверху вниз (элемент появится снизу от родителя)
    var _top2 = fixed ? parentRect.top + parentRect.height + margins.y : parentRect.height + margins.y;
    if (align === 'top') {
      _top2 = fixed ? parentRect.top - element.offsetHeight - margins.y : -element.offsetHeight - margins.y;
    }
    element.style.top = "".concat(_top2, "px");
    if (float === 'start') {
      // Элемент установится по левому краю с отступом
      var _left = fixed ? parentRect.left + margins.x : margins.x;
      element.style.left = "".concat(_left, "px");
      element.style.right = ''; // --
    }
    if (float === 'center') {
      // Элемент установится по центру без отступа, в зависимости от соотношения размеров родителя и списка в плоскости соприкосновения
      var _shift = (parentRect.width - element.offsetWidth) / 2;
      var _left2 = fixed ? parentRect.left + _shift : _shift;
      element.style.left = "".concat(_left2, "px");
      element.style.right = ''; // --
    }
    if (float === 'end') {
      // Элемент установится по правому краю с отступом
      if (fixed) {
        element.style.left = "".concat(parentRect.left - element.offsetWidth + parentRect.width - margins.x, "px");
        element.style.right = ''; // --
      } else {
        element.style.right = "".concat(-margins.x, "px");
        element.style.left = ''; // --
      }
    }
  }
};

// Устанавливаем правильную начальную позицию по координатам
var setInitPositionByCoords = function setInitPositionByCoords(element, coords, margins) {
  element.style.top = "".concat(coords.y + margins.y, "px");
  element.style.left = "".concat(coords.x + margins.x, "px");
};

// Финальное выравнивание и чистка
var fixFloat = function fixFloat(element, parentRect, overflowRect, margins, align, fixed) {
  var elementRect = element.getBoundingClientRect();
  if (align === 'vertical' || align === 'left' || align === 'right') {
    var left = fixed ? elementRect.left + elementRect.width : parentRect.left + elementRect.width + margins.x;
    if (left > overflowRect.right) {
      element.style.left = "".concat(fixed ? parentRect.right - elementRect.width - margins.x : parentRect.width - margins.x - elementRect.width, "px");
    }
  } else if (elementRect.top + elementRect.height > overflowRect.bottom) {
    element.style.top = "".concat(fixed ? elementRect.top - parentRect.height + margins.y : margins.y, "px");
  }
};

// Проверяем пересечение
var resetPositionByParent = function resetPositionByParent(element, parentRect, align, float, fixed, container) {
  var margins = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : {
    x: 0,
    y: 0
  };
  // Перекрывающий элемент
  var overflowElement = container !== null && container !== void 0 ? container : getScrollParent(element);
  // Габариты перекрывающего элемента
  var overflowElementRect = fixed ?
  // При fixed считаем от границ окна, а не body
  {
    top: 0,
    left: 0,
    width: window.innerWidth,
    height: window.innerHeight,
    bottom: window.innerHeight,
    right: window.innerWidth
  } : overflowElement.getBoundingClientRect();
  // Габариты выпадающего элемента
  var rect = function rect() {
    return element.getBoundingClientRect();
  };
  if (align === 'horizontal' || align === 'left' || align === 'right') {
    var rectWidth = float === 'center' ? rect().width / 2 : rect().width;
    // Если элемент ушел за правый край контейнера
    if (parentRect.right + rectWidth > overflowElementRect.right) {
      // перебрасываем его влево
      var right = fixed ? parentRect.right + margins.x : parentRect.width + margins.x;
      element.style.right = "".concat(right, "px");
      element.style.left = ''; // очищаем неопределенность
    }
    // Если элемент ушел за левый край контейнера
    if (parentRect.left - rectWidth < overflowElementRect.left) {
      // перекидываем его вправо
      var left = fixed ? parentRect.left + margins.x : parentRect.width + margins.x;
      element.style.left = "".concat(left, "px");
      element.style.right = ''; // --
    }
  }
  var rectHeight = float === 'center' && align !== 'vertical' ? rect().height / 2 : rect().height;
  // Если элемент ушел ниже нижней границы контейнера
  if (parentRect.bottom + rectHeight > overflowElementRect.bottom) {
    if (align === 'vertical' || align === 'top' || align === 'bottom') {
      // Перекидываем его ввеерх
      var top = fixed ? parentRect.top - rect().height - margins.y : -rect().height - margins.y;
      element.style.top = "".concat(top, "px");
    } else {
      var _top3 = fixed ? parentRect.top - rect().height + parentRect.height + margins.y : -rect().height + parentRect.height + margins.y;
      element.style.top = "".concat(_top3, "px");
    }
  }

  // Если компонент ушел выше верхней границы
  if (rect().top < overflowElementRect.top) {
    // Перерисовываем его вниз
    var _top4 = fixed ? parentRect.top + Number(parentRect.height) + margins.y : Number(parentRect.height) + margins.y;
    element.style.top = "".concat(_top4, "px");
    element.style.bottom = '';
  }

  // Ровняем компонент
  fixFloat(element, parentRect, overflowElementRect, margins, align, fixed);
};

// Проверяем пересечение с контейнером по координатам
var resetPositionByCoords = function resetPositionByCoords(element, coords) {
  var margins = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {
    x: 0,
    y: 0
  };
  var rect = element.getBoundingClientRect();
  if (coords.x + rect.width > window.innerWidth) {
    element.style.left = "".concat(coords.x - rect.width - margins.x, "px");
    element.style.right = '';
  } else if (coords.x - rect.width < 0) {
    element.style.left = "".concat(coords.x + margins.x, "px");
    element.style.right = '';
  }
  if (coords.y + rect.height > window.innerHeight) {
    element.style.top = "".concat(coords.y - rect.height - margins.y, "px");
  } else if (coords.y - rect.height < 0) {
    element.style.top = "".concat(coords.y + margins.y, "px");
  }
};
var setElement = function setElement(_ref) {
  var dropRef = _ref.dropRef,
    align = _ref.align,
    float = _ref.float,
    margins = _ref.margins,
    fixed = _ref.fixed,
    coords = _ref.coords,
    container = _ref.container,
    parentRef = _ref.parentRef,
    scrollHandler = _ref.scrollHandler,
    fitListToParent = _ref.fitListToParent,
    autoFixPosition = _ref.autoFixPosition;
  var drop = dropRef.current;
  if (drop) {
    var _parentRef$current$pa, _parentRef$current;
    // получаем родителя, относительно которого будем вести все расчеты
    var parent = (_parentRef$current$pa = parentRef === null || parentRef === void 0 ? void 0 : (_parentRef$current = parentRef.current) === null || _parentRef$current === void 0 ? void 0 : _parentRef$current.parentElement) !== null && _parentRef$current$pa !== void 0 ? _parentRef$current$pa : drop.parentElement;
    if (parent) {
      // Делаем список невидимым
      drop.style.opacity = '0';

      // Получаем габариты родителя
      var parentRect = parent.getBoundingClientRect();

      // Устанавливаем position в зависимости от выбранной стратегии
      if (fixed) {
        drop.style.position = 'fixed';
        if (fitListToParent) {
          drop.style.minWidth = "".concat(parent.offsetWidth, "px");
          drop.style.maxWidth = "".concat(parent.offsetWidth, "px");
          drop.style.width = "".concat(parent.offsetWidth, "px");
        }
      } else {
        drop.style.position = 'absolute';
        drop.style.minWidth = "";
        drop.style.maxWidth = "";
        drop.style.width = '';
      }

      // Принудительный сброс позиции, если задана
      drop.style.top = '';
      drop.style.left = '';
      drop.style.bottom = '';
      drop.style.right = '';

      // Производим расчеты относительно координат или родителя
      if (coords) {
        setInitPositionByCoords(drop, coords, margins); // Первичная простановка позиции, чтобы начать расчеты
        autoFixPosition && resetPositionByCoords(drop, coords, margins); // Перерасчет позиции
      } else {
        setInitPositionByParent(drop, parentRect, float, align, margins, fixed); // Первичная простановка позиции, чтобы начать расчеты
        autoFixPosition && resetPositionByParent(drop, parentRect, align, float, fixed, container, margins); // Перерасчет позиции относительно контейнера
      }

      // Ловим событие scroll, если надо, на всех родителях, которые его поддерживают
      if (scrollHandler) {
        var scrollable = getScrollParents(parent);
        scrollable.forEach(function () {
          return document.body.addEventListener('scroll', scrollHandler);
        });
      }

      // Показываем выпавший и спозиционированный элемент
      drop.style.opacity = '1';
    }
  }
};
// Оптимизация скрола
var updateElement = function updateElement(_ref2) {
  var dropRef = _ref2.dropRef,
    align = _ref2.align,
    float = _ref2.float,
    margins = _ref2.margins,
    fixed = _ref2.fixed,
    coords = _ref2.coords,
    container = _ref2.container,
    parentRef = _ref2.parentRef,
    autoFixPosition = _ref2.autoFixPosition,
    callbacks = _ref2.callbacks;
  var drop = dropRef.current;
  if (drop) {
    var _parentRef$current$pa2, _parentRef$current2;
    // получаем родителя, относительно которого будем вести все расчеты
    var parent = (_parentRef$current$pa2 = parentRef === null || parentRef === void 0 ? void 0 : (_parentRef$current2 = parentRef.current) === null || _parentRef$current2 === void 0 ? void 0 : _parentRef$current2.parentElement) !== null && _parentRef$current$pa2 !== void 0 ? _parentRef$current$pa2 : drop.parentElement;
    var parentRect = parent.getBoundingClientRect();
    if (coords) {
      setInitPositionByCoords(drop, coords, margins); // Первичная простановка позиции, чтобы начать расчеты
      autoFixPosition && resetPositionByCoords(drop, coords, margins); // Перерасчет позиции
    } else {
      setInitPositionByParent(drop, parentRect, float, align, margins, fixed); // Первичная простановка позиции, чтобы начать расчеты
      autoFixPosition && resetPositionByParent(drop, parentRect, align, float, fixed, container, margins); // Перерасчет позиции относительно контейнера
    }

    // Проверяем не исчез ли из области видимости якорный компонент
    var scrollable = getScrollParent(parent);
    if (!isElementInViewport(parent, scrollable)) {
      if (typeof (callbacks === null || callbacks === void 0 ? void 0 : callbacks.onHide) === 'function') {
        callbacks === null || callbacks === void 0 ? void 0 : callbacks.onHide();
      }
    }
  }
};

/*
    Хук для создания выпадающих областей.

    Для совместимости с IOS изначальное состояние элемента задается невидимым через opacity, а не скрытым через visibility, так как последнее приводит к неработающему скроллу.

    align - определяет направление выпадания списка
        horisontal - по горизонтальной оси
        vertical - по вертикальной оси
    
    float - определяет выравниваниее компонента при выпадании
        start - по верхнему краю для horizontal | по левому краю для vertical
        end - по нижнему краю для horizontal | по правому краю для vertical

    margins - дополнительные отступы от родительского контейнера

    fixed - определяет стратегию выпадания относительно родительского элемента или относительно всего окна

    coords - работают в паре с fixed - позволяют передать якорную точку для выпадающего элемента

    container - позволяет вручную определить контейнер, относительно которого будут вестись расчеты пересечений с краями

    parentRef - ссылка на родительский элемент

    followParentWhenScroll - позволяет элементу оставаться для пользователя зрительно на фиксированном месте страницы (а не экрана)
    событие скролла навешивается на все родительские элементы, которые его поддерживают, если элемент уходит из области видимости любого из родителей список прячется

    callbacks - события генерируемы хуком
*/

export var useDrop = function useDrop(config) {
  var _config$align = config.align,
    align = _config$align === void 0 ? 'vertical' : _config$align,
    _config$float = config.float,
    float = _config$float === void 0 ? 'start' : _config$float,
    _config$margins = config.margins,
    margins = _config$margins === void 0 ? {
      x: 0,
      y: 0
    } : _config$margins,
    _config$fixed = config.fixed,
    fixed = _config$fixed === void 0 ? false : _config$fixed,
    coords = config.coords,
    container = config.container,
    parentRef = config.parentRef,
    followParentWhenScroll = config.followParentWhenScroll,
    callbacks = config.callbacks,
    fitListToParent = config.fitListToParent,
    _config$autoFixPositi = config.autoFixPosition,
    autoFixPosition = _config$autoFixPositi === void 0 ? true : _config$autoFixPositi;
  var dropRef = useRef(null);
  var debouncerRender = useDebounce(setElement, 0); // Задержка рендера (так как приложение может быть сильно нагружено, повышаем шанс того, что все будет отрендерено)
  var debouncerScroll = useDebounce(updateElement, 0); // Задержка скрола

  // Метод используется в событии scroll
  var scrollHandler = useCallback(function () {
    return debouncerScroll({
      dropRef: dropRef,
      align: align,
      float: float,
      margins: margins,
      fixed: fixed,
      coords: coords,
      container: container,
      parentRef: parentRef,
      autoFixPosition: autoFixPosition,
      callbacks: callbacks
    });
  }, [debouncerScroll, align, margins, float, fixed, coords, container, parentRef, callbacks, autoFixPosition]);
  useEffect(function () {
    // Вызываем установку позиции при появлении элемента
    debouncerRender({
      dropRef: dropRef,
      align: align,
      float: float,
      margins: margins,
      fixed: fixed,
      coords: coords,
      container: container,
      parentRef: parentRef,
      scrollHandler: followParentWhenScroll ? scrollHandler : undefined,
      fitListToParent: fitListToParent,
      autoFixPosition: autoFixPosition
    });
    var drop = dropRef.current;
    var parentElement = drop === null || drop === void 0 ? void 0 : drop.parentElement;
    var overflowElement = container !== null && container !== void 0 ? container : (parentRef === null || parentRef === void 0 ? void 0 : parentRef.current) && getScrollParent(parentRef.current);
    var resizeObserver;

    // Включаем слежение за изменением размера родителя (все кроме IE)
    if (window.ResizeObserver && overflowElement) {
      resizeObserver = new ResizeObserver(function () {
        scrollHandler();
      });
      resizeObserver.observe(overflowElement);
    }
    return function () {
      // Выключаем событие скрола при анмаунте
      if (followParentWhenScroll) {
        var parent = container !== null && container !== void 0 ? container : parentElement;
        var scrollable = getScrollParents(parent);
        scrollable.forEach(function (s) {
          return s.removeEventListener('scroll', scrollHandler);
        });
      }

      // Выключаем слежение за размерами родителя
      if (resizeObserver && overflowElement) {
        resizeObserver.unobserve(overflowElement);
      }
    };
  }, [debouncerRender, scrollHandler, dropRef, align, float, margins, fixed, coords, container, followParentWhenScroll, parentRef, fitListToParent, autoFixPosition]);
  return dropRef; // ссылка на управляемый элемент должна быть присвоена выпадающему элементу
};